
using namespace System.Management.Automation
using namespace System.Management.Automation.Language

Register-ArgumentCompleter -Native -CommandName 'gpg-sq' -ScriptBlock {
    param($wordToComplete, $commandAst, $cursorPosition)

    $commandElements = $commandAst.CommandElements
    $command = @(
        'gpg-sq'
        for ($i = 1; $i -lt $commandElements.Count; $i++) {
            $element = $commandElements[$i]
            if ($element -isnot [StringConstantExpressionAst] -or
                $element.StringConstantType -ne [StringConstantType]::BareWord -or
                $element.Value.StartsWith('-') -or
                $element.Value -eq $wordToComplete) {
                break
        }
        $element.Value
    }) -join ';'

    $completions = @(switch ($command) {
        'gpg-sq' {
            [CompletionResult]::new('-s', '-s', [CompletionResultType]::ParameterName, 'make a signature')
            [CompletionResult]::new('--sign', '--sign', [CompletionResultType]::ParameterName, 'make a signature')
            [CompletionResult]::new('--clear-sign', '--clear-sign', [CompletionResultType]::ParameterName, 'make a clear text signature')
            [CompletionResult]::new('-b', '-b', [CompletionResultType]::ParameterName, 'make a detached signature')
            [CompletionResult]::new('--detach-sign', '--detach-sign', [CompletionResultType]::ParameterName, 'make a detached signature')
            [CompletionResult]::new('-e', '-e', [CompletionResultType]::ParameterName, 'encrypt data')
            [CompletionResult]::new('--encrypt', '--encrypt', [CompletionResultType]::ParameterName, 'encrypt data')
            [CompletionResult]::new('-c', '-c', [CompletionResultType]::ParameterName, 'encryption only with symmetric cipher')
            [CompletionResult]::new('--symmetric', '--symmetric', [CompletionResultType]::ParameterName, 'encryption only with symmetric cipher')
            [CompletionResult]::new('-d', '-d', [CompletionResultType]::ParameterName, 'decrypt data (default)')
            [CompletionResult]::new('--decrypt', '--decrypt', [CompletionResultType]::ParameterName, 'decrypt data (default)')
            [CompletionResult]::new('--verify', '--verify', [CompletionResultType]::ParameterName, 'verify a signature')
            [CompletionResult]::new('-k', '-k', [CompletionResultType]::ParameterName, 'list keys')
            [CompletionResult]::new('--list-keys', '--list-keys', [CompletionResultType]::ParameterName, 'list keys')
            [CompletionResult]::new('--list-signatures', '--list-signatures', [CompletionResultType]::ParameterName, 'list keys and signatures')
            [CompletionResult]::new('--check-signatures', '--check-signatures', [CompletionResultType]::ParameterName, 'list and check key signatures')
            [CompletionResult]::new('--fingerprint', '--fingerprint', [CompletionResultType]::ParameterName, 'list keys and fingerprints')
            [CompletionResult]::new('-K', '-K ', [CompletionResultType]::ParameterName, 'list secret keys')
            [CompletionResult]::new('--list-secret-keys', '--list-secret-keys', [CompletionResultType]::ParameterName, 'list secret keys')
            [CompletionResult]::new('--generate-key', '--generate-key', [CompletionResultType]::ParameterName, 'generate a new key pair')
            [CompletionResult]::new('--quick-generate-key', '--quick-generate-key', [CompletionResultType]::ParameterName, 'quickly generate a new key pair')
            [CompletionResult]::new('--quick-add-uid', '--quick-add-uid', [CompletionResultType]::ParameterName, 'quickly add a new user-id')
            [CompletionResult]::new('--quick-revoke-uid', '--quick-revoke-uid', [CompletionResultType]::ParameterName, 'quickly revoke a user-id')
            [CompletionResult]::new('--quick-set-expire', '--quick-set-expire', [CompletionResultType]::ParameterName, 'quickly set a new expiration date')
            [CompletionResult]::new('--full-generate-key', '--full-generate-key', [CompletionResultType]::ParameterName, 'full featured key pair generation')
            [CompletionResult]::new('--generate-revocation', '--generate-revocation', [CompletionResultType]::ParameterName, 'generate a revocation certificate')
            [CompletionResult]::new('--delete-keys', '--delete-keys', [CompletionResultType]::ParameterName, 'remove keys from the public keyring')
            [CompletionResult]::new('--delete-secret-keys', '--delete-secret-keys', [CompletionResultType]::ParameterName, 'remove keys from the secret keyring')
            [CompletionResult]::new('--quick-sign-key', '--quick-sign-key', [CompletionResultType]::ParameterName, 'quickly sign a key')
            [CompletionResult]::new('--quick-lsign-key', '--quick-lsign-key', [CompletionResultType]::ParameterName, 'quickly sign a key locally')
            [CompletionResult]::new('--quick-revoke-sig', '--quick-revoke-sig', [CompletionResultType]::ParameterName, 'quickly revoke a key signature')
            [CompletionResult]::new('--sign-key', '--sign-key', [CompletionResultType]::ParameterName, 'sign a key')
            [CompletionResult]::new('--lsign-key', '--lsign-key', [CompletionResultType]::ParameterName, 'sign a key locally')
            [CompletionResult]::new('--edit-key', '--edit-key', [CompletionResultType]::ParameterName, 'sign or edit a key')
            [CompletionResult]::new('--change-passphrase', '--change-passphrase', [CompletionResultType]::ParameterName, 'change a passphrase')
            [CompletionResult]::new('--export', '--export', [CompletionResultType]::ParameterName, 'export keys')
            [CompletionResult]::new('--send-keys', '--send-keys', [CompletionResultType]::ParameterName, 'export keys to a keyserver')
            [CompletionResult]::new('--receive-keys', '--receive-keys', [CompletionResultType]::ParameterName, 'import keys from a keyserver')
            [CompletionResult]::new('--search-keys', '--search-keys', [CompletionResultType]::ParameterName, 'search for keys on a keyserver')
            [CompletionResult]::new('--refresh-keys', '--refresh-keys', [CompletionResultType]::ParameterName, 'update all keys from a keyserver')
            [CompletionResult]::new('--import', '--import', [CompletionResultType]::ParameterName, 'import/merge keys')
            [CompletionResult]::new('--update-trustdb', '--update-trustdb', [CompletionResultType]::ParameterName, 'update the trust database')
            [CompletionResult]::new('--print-md', '--print-md', [CompletionResultType]::ParameterName, 'print message digests')
            [CompletionResult]::new('--server', '--server', [CompletionResultType]::ParameterName, 'run in server mode')
            [CompletionResult]::new('--tofu-policy', '--tofu-policy', [CompletionResultType]::ParameterName, 'set the TOFU policy for a key')
            [CompletionResult]::new('--x-sequoia-parcimonie', '--x-sequoia-parcimonie', [CompletionResultType]::ParameterName, 'continuously update certificates')
            [CompletionResult]::new('-v', '-v', [CompletionResultType]::ParameterName, 'verbose')
            [CompletionResult]::new('--verbose', '--verbose', [CompletionResultType]::ParameterName, 'verbose')
            [CompletionResult]::new('-q', '-q', [CompletionResultType]::ParameterName, 'be somewhat more quiet')
            [CompletionResult]::new('--quiet', '--quiet', [CompletionResultType]::ParameterName, 'be somewhat more quiet')
            [CompletionResult]::new('--options', '--options', [CompletionResultType]::ParameterName, 'read options from FILE')
            [CompletionResult]::new('--log-file', '--log-file', [CompletionResultType]::ParameterName, 'write server mode logs to FILE')
            [CompletionResult]::new('--default-key', '--default-key', [CompletionResultType]::ParameterName, 'use NAME as default secret key')
            [CompletionResult]::new('--encrypt-to', '--encrypt-to', [CompletionResultType]::ParameterName, 'encrypt to user ID NAME as well')
            [CompletionResult]::new('--group', '--group', [CompletionResultType]::ParameterName, 'set up email aliases')
            [CompletionResult]::new('--openpgp', '--openpgp', [CompletionResultType]::ParameterName, 'use strict OpenPGP behavior')
            [CompletionResult]::new('-n', '-n', [CompletionResultType]::ParameterName, 'do not make any changes')
            [CompletionResult]::new('--dry-run', '--dry-run', [CompletionResultType]::ParameterName, 'do not make any changes')
            [CompletionResult]::new('-i', '-i', [CompletionResultType]::ParameterName, 'prompt before overwriting')
            [CompletionResult]::new('--interactive', '--interactive', [CompletionResultType]::ParameterName, 'prompt before overwriting')
            [CompletionResult]::new('-a', '-a', [CompletionResultType]::ParameterName, 'create ascii armored output')
            [CompletionResult]::new('--armor', '--armor', [CompletionResultType]::ParameterName, 'create ascii armored output')
            [CompletionResult]::new('-o', '-o', [CompletionResultType]::ParameterName, 'write output to FILE')
            [CompletionResult]::new('--output', '--output', [CompletionResultType]::ParameterName, 'write output to FILE')
            [CompletionResult]::new('--textmode', '--textmode', [CompletionResultType]::ParameterName, 'use canonical text mode')
            [CompletionResult]::new('-z', '-z', [CompletionResultType]::ParameterName, 'set compress level to N (0 disables)')
            [CompletionResult]::new('--auto-key-locate', '--auto-key-locate', [CompletionResultType]::ParameterName, 'use MECHANISMS to locate keys by mail address')
            [CompletionResult]::new('--auto-key-import', '--auto-key-import', [CompletionResultType]::ParameterName, 'import missing key from a signature')
            [CompletionResult]::new('--include-key-block', '--include-key-block', [CompletionResultType]::ParameterName, 'include the public key in signatures')
            [CompletionResult]::new('--disable-dirmngr', '--disable-dirmngr', [CompletionResultType]::ParameterName, 'disable all access to the dirmngr')
            [CompletionResult]::new('-r', '-r', [CompletionResultType]::ParameterName, 'encrypt for USER-ID')
            [CompletionResult]::new('--recipient', '--recipient', [CompletionResultType]::ParameterName, 'encrypt for USER-ID')
            [CompletionResult]::new('-u', '-u', [CompletionResultType]::ParameterName, 'use USER-ID to sign or decrypt')
            [CompletionResult]::new('--local-user', '--local-user', [CompletionResultType]::ParameterName, 'use USER-ID to sign or decrypt')
            [CompletionResult]::new('--x-sequoia-autostart-parcimonie', '--x-sequoia-autostart-parcimonie', [CompletionResultType]::ParameterName, 'automatically start daemon to update certs')
            [CompletionResult]::new('-h', '-h', [CompletionResultType]::ParameterName, 'Print help (see more with ''--help'')')
            [CompletionResult]::new('--help', '--help', [CompletionResultType]::ParameterName, 'Print help (see more with ''--help'')')
            [CompletionResult]::new('-V', '-V ', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('--version', '--version', [CompletionResultType]::ParameterName, 'Print version')
            break
        }
    })

    $completions.Where{ $_.CompletionText -like "$wordToComplete*" } |
        Sort-Object -Property ListItemText
}
