
using namespace System.Management.Automation
using namespace System.Management.Automation.Language

Register-ArgumentCompleter -Native -CommandName 'gpgv-sq' -ScriptBlock {
    param($wordToComplete, $commandAst, $cursorPosition)

    $commandElements = $commandAst.CommandElements
    $command = @(
        'gpgv-sq'
        for ($i = 1; $i -lt $commandElements.Count; $i++) {
            $element = $commandElements[$i]
            if ($element -isnot [StringConstantExpressionAst] -or
                $element.StringConstantType -ne [StringConstantType]::BareWord -or
                $element.Value.StartsWith('-') -or
                $element.Value -eq $wordToComplete) {
                break
        }
        $element.Value
    }) -join ';'

    $completions = @(switch ($command) {
        'gpgv-sq' {
            [CompletionResult]::new('-v', '-v', [CompletionResultType]::ParameterName, 'verbose')
            [CompletionResult]::new('--verbose', '--verbose', [CompletionResultType]::ParameterName, 'verbose')
            [CompletionResult]::new('-q', '-q', [CompletionResultType]::ParameterName, 'be somewhat more quiet')
            [CompletionResult]::new('--quiet', '--quiet', [CompletionResultType]::ParameterName, 'be somewhat more quiet')
            [CompletionResult]::new('--keyring', '--keyring', [CompletionResultType]::ParameterName, 'take the keys from the keyring FILE')
            [CompletionResult]::new('-o', '-o', [CompletionResultType]::ParameterName, 'write output to FILE')
            [CompletionResult]::new('--output', '--output', [CompletionResultType]::ParameterName, 'write output to FILE')
            [CompletionResult]::new('--ignore-time-conflict', '--ignore-time-conflict', [CompletionResultType]::ParameterName, 'make timestamp conflicts only a warning')
            [CompletionResult]::new('--status-fd', '--status-fd', [CompletionResultType]::ParameterName, 'write status info to this FD')
            [CompletionResult]::new('--weak-digest', '--weak-digest', [CompletionResultType]::ParameterName, 'reject signatures made with ALGO')
            [CompletionResult]::new('-h', '-h', [CompletionResultType]::ParameterName, 'Print help (see more with ''--help'')')
            [CompletionResult]::new('--help', '--help', [CompletionResultType]::ParameterName, 'Print help (see more with ''--help'')')
            [CompletionResult]::new('-V', '-V ', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('--version', '--version', [CompletionResultType]::ParameterName, 'Print version')
            break
        }
    })

    $completions.Where{ $_.CompletionText -like "$wordToComplete*" } |
        Sort-Object -Property ListItemText
}
